@@ -1,88 +1,33 @@
 package ZeroMQ;
-use 5.008;
 use strict;
-
-our $VERSION = '0.02';
-our @ISA = qw(Exporter);
-
-# TODO: keep in sync with docs below and Makefile.PL
-
-our %EXPORT_TAGS = (
-# socket types
-    socket => [ qw(
-        ZMQ_PAIR
-        ZMQ_PUB
-        ZMQ_SUB
-        ZMQ_REQ
-        ZMQ_REP
-        ZMQ_XREQ
-        ZMQ_XREP
-        ZMQ_PULL
-        ZMQ_PUSH
-        ZMQ_UPSTREAM
-        ZMQ_DOWNSTREAM
-    ),
-# socket send/recv flags
-    qw(
-        ZMQ_NOBLOCK
-        ZMQ_SNDMORE
-    ),
-# get/setsockopt options
-    qw(
-        ZMQ_HWM
-        ZMQ_SWAP
-        ZMQ_AFFINITY
-        ZMQ_IDENTITY
-        ZMQ_SUBSCRIBE
-        ZMQ_UNSUBSCRIBE
-        ZMQ_RATE
-        ZMQ_RECOVERY_IVL
-        ZMQ_MCAST_LOOP
-        ZMQ_SNDBUF
-        ZMQ_RCVBUF
-        ZMQ_RCVMORE
-    ),
-# i/o multiplexing
-    qw(
-        ZMQ_POLLIN
-        ZMQ_POLLOUT
-        ZMQ_POLLERR
-    ),
-    ],
-# devices
-    device => [ qw(
-        ZMQ_QUEUE
-        ZMQ_FORWARDER
-        ZMQ_STREAMER
-    ), ],
-# max size of vsm message
-    message => [ qw(
-        ZMQ_MAX_VSM_SIZE
-    ),
-# message types
-    qw(
-        ZMQ_DELIMITER
-        ZMQ_VSM
-    ),
-# message flags
-    qw(
-        ZMQ_MSG_MORE
-        ZMQ_MSG_SHARED
-    ), ]
-);
-
-$EXPORT_TAGS{all} = [ map { @$_ } values %EXPORT_TAGS ];
-our @EXPORT_OK = ( 'ZMQ_HAUSNUMERO', @{ $EXPORT_TAGS{'all'} } );
-our @EXPORT = qw();
-
-require XSLoader;
-XSLoader::load('ZeroMQ', $VERSION);
+BEGIN {
+    our $VERSION = '0.03';
+    our @ISA = qw(Exporter);
+}
+use ZeroMQ::Raw ();
+use ZeroMQ::Context;
+use ZeroMQ::Socket;
+use ZeroMQ::Message;
+use ZeroMQ::Constants;
+use 5.008;
+use Carp ();
+use IO::Handle;
 
 our %SERIALIZERS;
 our %DESERIALIZERS;
 sub register_read_type { $DESERIALIZERS{$_[0]} = $_[1] }
 sub register_write_type { $SERIALIZERS{$_[0]} = $_[1] }
 
+sub import {
+    my $class = shift;
+    if (@_) {
+        ZeroMQ::Constants->export_to_level( 1, $class, @_ );
+    }
+}
+
+sub _get_serializer { $SERIALIZERS{$_[1]} }
+sub _get_deserializer { $DESERIALIZERS{$_[1]} }
+
 eval {
     require JSON;
     JSON->import(2.00);
@@ -90,11 +35,6 @@ eval {
     register_write_type(json => \&JSON::encode_json);
 };
 
-
-sub ZeroMQ::Context::socket {
-    return ZeroMQ::Socket->new(@_); # $_[0] should contain the context
-}
-
 1;
 __END__
 
@@ -102,7 +42,7 @@ __END__
 
 ZeroMQ - A ZeroMQ2 wrapper for Perl
 
-=head1 SYNOPSIS
+=head1 SYNOPSIS ( HIGH-LEVEL API )
 
     # echo server
     use ZeroMQ qw/:all/;
@@ -117,14 +57,40 @@ ZeroMQ - A ZeroMQ2 wrapper for Perl
         $sock->send($msg);
     }
 
+    # json (if JSON.pm is available)
+    $sock->send_as( json => { foo => "bar" } );
+    my $thing = $sock->recv_as( "json" );
+
     # custom serialization
     ZeroMQ::register_read_type(myformat => sub { ... });
     ZeroMQ::register_write_type(myformat => sub { .. });
 
-    $socket->send_as( myformat => $data ); # serialize using above callback
-    my $thing = $socket->recv_as( "myformat" );
+    $sock->send_as( myformat => $data ); # serialize using above callback
+    my $thing = $sock->recv_as( "myformat" );
 
-See the F<eg/> directory for full examples.
+=head1 SYNOPSIS ( LOW-LEVEL API )
+
+    use ZeroMQ::Raw;
+
+    my $ctxt = zmq_init($threads);
+    my $rv   = zmq_term($ctxt);
+
+    my $msg  = zmq_msg_init();
+    my $msg  = zmq_msg_init_size( $size );
+    my $msg  = zmq_msg_init_data( $data );
+    my $rv   = zmq_msg_close( $msg );
+    my $rv   = zmq_msg_move( $dest, $src );
+    my $rv   = zmq_msg_copy( $dest, $src );
+    my $data = zmq_msg_data( $msg );
+    my $size = zmq_msg_size( $msg);
+
+    my $sock = zmq_socket( $ctxt, $type );
+    my $rv   = zmq_close( $sock );
+    my $rv   = zmq_setsockopt( $socket, $option, $value );
+    my $val  = zmq_getsockopt( $socket, $option );
+    my $rv   = zmq_bind( $sock, $addr );
+    my $rv   = zmq_send( $sock, $msg, $flags );
+    my $msg  = zmq_recv( $sock, $flags );
 
 =head1 DESCRIPTION
 
@@ -132,9 +98,33 @@ The C<ZeroMQ> module is a wrapper of the 0MQ message passing library for Perl.
 It's a thin wrapper around the C API. Please read L<http://zeromq.org> for
 more details on ZeroMQ.
 
+=head1 CLASS WALKTHROUGH
+
+=over 4
+
+=item ZeroMQ::Raw
+
+Use L<ZeroMQ::Raw> to get access to the C API such as C<zmq_init>, C<zmq_socket>, et al. Functions provided in this low level API should follow the C API exactly.
+
+=item ZeroMQ::Constants
+
+L<ZeroMQ::Constants> contains all of the constants that are known to be extractable from zmq.h. Do note that sometimes the list changes due to additions/deprecations in the underlying zeromq2 library. We try to do our best to make things available (at least to warn you that some symbols are deprecated), but it may not always be possible.
+
+=item ZeroMQ::Context
+
+=item ZeroMQ::Socket
+
+=item ZeroMQ::Message
+
+L<ZeroMQ::Context>, L<ZeroMQ::Socket>, L<ZeroMQ::Message> contain the high-level, more perl-ish interface to the zeromq functionalities.
+
+=item ZeroMQ
+
 Loading C<ZeroMQ> will make the L<ZeroMQ::Context>, L<ZeroMQ::Socket>, and 
 L<ZeroMQ::Message> classes available as well.
 
+=back
+
 =head1 BASIC USAGE
 
 To start using ZeroMQ, you need to create a context object, then as many ZeroMQ::Socket as you need:
@@ -142,6 +132,16 @@ To start using ZeroMQ, you need to create a context object, then as many ZeroMQ:
     my $ctxt = ZeroMQ::Context->new;
     my $socket = $ctxt->socket( ... options );
 
+You need to call C<bind()> or C<connect()> on the socket, depending on your usage. For example on a typical server-client model you would write on the server side:
+
+    $socket->bind( "tcp://127.0.0.1:9999" );
+
+and on the client side:
+
+    $socket->connect( "tcp://127.0.0.1:9999" );
+
+The underlying zeromq library offers TCP, multicast, in-process, and ipc connection patterns. Read the zeromq manual for more details on other ways to setup the socket.
+
 When sending data, you can either pass a ZeroMQ::Message object or a Perl string. 
 
     # the following two send() calls are equivalent
@@ -155,7 +155,9 @@ To receive, simply call C<recv()> on the socket
 
     my $msg = $socket->recv;
 
-The received message is an instance of ZeroMQ::Message object.
+The received message is an instance of ZeroMQ::Message object, and you can access the content held in the message via the C<data()> method:
+
+    my $data = $msg->data;
 
 =head1 SERIALIZATION
 
@@ -163,14 +165,15 @@ ZeroMQ.pm comes with a simple serialization/deserialization mechanism.
 
 To serialize, use C<register_write_type()> to register a name and an
 associated callback to serialize the data. For example, for JSON we do
-the following:
+the following (this is already done for you in ZeroMQ.pm if you have
+JSON.pm installed):
 
     use JSON ();
     ZeroMQ::register_write_type('json' => \&JSON::encode_json);
     ZeroMQ::register_read_type('json' => \&JSON::decode_json);
 
-Then you can use C<send_as()> and C<recv_as()> to specify the serialization type as the
-first argument:
+Then you can use C<send_as()> and C<recv_as()> to specify the serialization 
+type as the first argument:
 
     my $ctxt = ZeroMQ::Context->new();
     my $sock = $ctxt->socket( ZMQ_REQ );
@@ -196,6 +199,47 @@ option, do something like this:
 Note that this will have a GLOBAL effect. If you want to change only
 your application, use a name that's different from 'json'.
 
+=head1 ASYNCHRONOUS I/O WITH ZEROMQ
+
+By default ZeroMQ comes with its own zmq_poll() mechanism that can handle
+non-blocking sockets. You can use this by creating calling zmq_poll with a list
+of hashrefs:
+
+    zmq_poll([
+        {
+            fd => fileno(STDOUT),
+            events => ZMQ_POLLOUT,
+            callback => \&callback,
+        },
+        {
+            socket => $zmq_socket,
+            events => ZMQ_POLLIN,
+            callback => \&callback
+        },
+    ], $timeout );
+
+Unfortunately this custom polling scheme doesn't play too well with AnyEvent.
+As of zeromq2-2.1.0, you can use getsockopt to retrieve the underlying file
+descriptor, so use that to integrate ZeroMQ and AnyEvent:
+
+    my $fh = zmq_getsockopt( $socket, ZMQ_FD );
+    my $w; $w = AE::io $fh, 0, sub {
+        undef $w;
+        print $fh "Hello\n";
+    };
+
+=head1 NOTES ON MULTI-PROCESS and MULTI-THREADED USAGE
+
+ZeroMQ works on both multi-process and multi-threaded use cases, but you need
+to be careful bout sharing ZeroMQ objects.
+
+For multi-process environments, you should not be sharing the context object.
+Create separate contexts for each process, and therefore you shouldn't
+be sharing the socket objects either.
+
+For multi-thread environemnts, you can share the same context object. However
+you cannot share sockets.
+
 =head1 FUNCTIONS
 
 =head2 version()
@@ -219,124 +263,10 @@ The callback receives the data received from the socket.
 Register a write callback for a given C<$name>. This is used in C<send_as()>
 The callback receives the Perl structure given to C<send_as()>
 
-=head1 EXPORTS
-
-You may choose to import one or more (using the C<:all> import tag)
-constants into your namespace by supplying arguments to the
-C<use ZeroMQ> call as shown in the synopsis above.
-
-The exportable constants are:
-
-=head1 EXPORTS
-
-=head2 C<:socket> - Socket types and socket options
-
-=over 4
-
-=item ZMQ_PAIR
-
-=item ZMQ_PUB
-
-=item ZMQ_SUB
-
-=item ZMQ_REQ
-
-=item ZMQ_REP
-
-=item ZMQ_XREQ
-
-=item ZMQ_XREP
-
-=item ZMQ_PULL
-
-=item ZMQ_PUSH
-
-=item ZMQ_UPSTREAM
-
-=item ZMQ_DOWNSTREAM
-
-=item ZMQ_NOBLOCK
-
-=item ZMQ_SNDMORE
-
-=item ZMQ_HWM
-
-=item ZMQ_SWAP
-
-=item ZMQ_AFFINITY
-
-=item ZMQ_IDENTITY
-
-=item ZMQ_SUBSCRIBE
-
-=item ZMQ_UNSUBSCRIBE
-
-=item ZMQ_RATE
-
-=item ZMQ_RECOVERY_IVL
-
-=item ZMQ_MCAST_LOOP
-
-=item ZMQ_SNDBUF
-
-=item ZMQ_RCVBUF
-
-=item ZMQ_RCVMORE
-
-=item ZMQ_POLLIN
-
-=item ZMQ_POLLOUT
-
-=item ZMQ_POLLERR
-
-=back
-
-=head2 C<:device> - Device types
-
-=over 4
-
-=item ZMQ_QUEUE
-
-=item ZMQ_FORWARDER
-
-=item ZMQ_STREAMER
-
-=back
-
-=head2 C<:message> - Message Options
-
-=over 4
-
-=item ZMQ_MAX_VSM_SIZE
-
-=item ZMQ_DELIMITER
-
-=item ZMQ_VSM
-
-=item ZMQ_MSG_MORE
-
-=item ZMQ_MSG_SHARED
-
-=back
-
-=head2 miscellaneous
-
-=over 4
-
-=item ZMQ_HAUSNUMERO
-
-=back
-
 =head1 CAVEATS
 
 This is an early release. Proceed with caution, please report
-(or better yet: fix) bugs you encounter. Tested againt 0MQ 2.0.7.
-
-Use of the C<inproc://> transport layer doesn't seem to work
-between two perl ithreads. This may be due to the fact that right now,
-context aren't shared between ithreads and C<inproc> works
-only within a single context. Try another transport layer until
-contexts can be shared.
+(or better yet: fix) bugs you encounter. Tested againt 0MQ 2.1.0.
 
 =head1 SEE ALSO
 
@@ -346,15 +276,15 @@ L<http://zeromq.org>
 
 =head1 AUTHOR
 
-Steffen Mueller, C<< <smueller@cpan.org> >>
-
 Daisuke Maki C<< <daisuke@endeworks.jp> >>
 
+Steffen Mueller, C<< <smueller@cpan.org> >>
+
 =head1 COPYRIGHT AND LICENSE
 
 The ZeroMQ module is
 
-Copyright (C) 2010 by Steffen Mueller
+Copyright (C) 2010 by Daisuke Maki
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.0 or,
